@@ -148,6 +148,29 @@ fcitx_config_widget_class_init(FcitxConfigWidgetClass *klass)
                                             G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
 }
 
+static void
+_fcitx_config_widget_reload(FcitxConfigWidget* widget, gpointer user_data)
+{
+    FcitxConfigWidget* config_widget = (FcitxConfigWidget*)user_data;
+    fcitx_config_widget_response(config_widget, CONFIG_WIDGET_RELOAD);
+}
+
+void fcitx_config_widget_connect(FcitxConfigWidget* self)
+{
+    GError* error = NULL;
+    self->improxy = fcitx_input_method_new(G_BUS_TYPE_SESSION,
+                                          G_DBUS_PROXY_FLAGS_NONE,
+                                          fcitx_utils_get_display_number(),
+                                          NULL,
+                                          &error
+                                         );
+    if (self->improxy == NULL) {
+        g_error_free(error);
+        return;
+    }
+    g_signal_connect(self->improxy, "ui-changed", G_CALLBACK(_fcitx_config_widget_reload), self);
+}
+
 static void
 fcitx_config_widget_init(FcitxConfigWidget *self)
 {
@@ -167,7 +190,6 @@ sync_hotkey(KeyGrabButton* button, gpointer user_data)
         keygrab_button_set_key(KEYGRAB_BUTTON(user_data), key1, mod1);
 }
 
-
 static void
 _fcitx_config_widget_hotkey_changed(KeyGrabButton* button, gpointer user_data)
 {
@@ -845,7 +867,6 @@ void fcitx_config_widget_response(
             fclose(fp);
 
             setConfigChangedFlag(TRUE);
-
             GError* error;
             gchar* argv[3];
             argv[0] = EXEC_PREFIX "/bin/fcitx-remote";
@@ -853,6 +874,13 @@ void fcitx_config_widget_response(
             argv[2] = 0;
             g_spawn_async(NULL, argv, NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, &error);
         }
+    } else if (action == CONFIG_WIDGET_RELOAD) {
+        FILE* fp = FcitxXDGGetFileUserWithPrefix(config_widget->prefix, config_widget->name, "r", NULL);
+
+        if (fp) {
+            dummy_config_load(config_widget->config,fp);
+            dummy_config_sync(config_widget->config);
+        }
     }
 }
 
@@ -928,6 +956,50 @@ gboolean fcitx_config_widget_response_cb(GtkDialog *dialog,
     return FALSE;
 }
 
+gboolean fcitx_config_widget_response_cb_exit(GtkDialog *dialog,
+        gint response,
+        gpointer user_data)
+{
+    if (response == GTK_RESPONSE_OK) {
+        FcitxConfigWidget* config_widget = (FcitxConfigWidget*) user_data;
+        fcitx_config_widget_response(config_widget, CONFIG_WIDGET_SAVE);
+    }
+    gtk_widget_destroy(GTK_WIDGET(dialog));
+    exit(0);
+    return FALSE;
+}
+
+int fcitx_utils_kill_fcitx_config_gtk3(FcitxAddon* addon) {
+    FILE *fp;
+    int pid;
+    char unused[150];
+    char buf[150];
+    char command[150];
+
+    sprintf(command,
+            "ps -ef | grep fcitx-config-gtk3 %s | grep -v grep",addon->name);
+
+    if ((fp = popen(command, "r")) == NULL)
+        return 0;
+
+    if ((fgets(buf, 150, fp)) != NULL) {
+        sscanf(buf, "%s\t%d\t%s", unused, &pid, unused);
+    }
+    pclose(fp);
+
+    if(pid == 32767)
+        return 0;
+
+    sprintf(command, "kill -9 %d", pid);
+
+    if ((fp = popen(command, "r")) == NULL)
+        return 0;
+
+    pclose(fp);
+    return 1;
+}
+
+
 GtkWidget* fcitx_config_dialog_new(FcitxAddon* addon, GtkWindow* parent)
 {
     gchar* config_desc_name = g_strdup_printf("%s.desc", addon->name);
@@ -954,8 +1026,16 @@ GtkWidget* fcitx_config_dialog_new(FcitxAddon* addon, GtkWindow* parent)
     g_free(config_file_name);
     gtk_widget_set_size_request(GTK_WIDGET(config_widget), -1, 400);
 
-    g_signal_connect(dialog, "response",
-                    G_CALLBACK(fcitx_config_widget_response_cb),
-                    config_widget);
+    if(parent != NULL){
+        g_signal_connect(dialog, "response",
+                        G_CALLBACK(fcitx_config_widget_response_cb),
+                        config_widget);
+    }
+    else {
+        g_signal_connect(dialog, "response",
+                        G_CALLBACK(fcitx_config_widget_response_cb_exit),
+                        config_widget);
+    }
+
     return dialog;
 }
