@@ -206,6 +206,7 @@ void _fcitx_im_widget_connect(FcitxImWidget* self)
         return;
     }
     g_signal_connect(self->improxy, "imlist-changed", G_CALLBACK(_fcitx_im_widget_imlist_changed_cb), self);
+//    g_signal_connect(self->improxy, "ui-changed", G_CALLBACK(_fcitx_im_widget_imlist_changed_cb), self);
 
     _fcitx_im_widget_load(self);
 }
@@ -231,8 +232,8 @@ void _fcitx_im_widget_load(FcitxImWidget* self)
 
         if (context.flag)
             gtk_tree_selection_select_iter(gtk_tree_view_get_selection(GTK_TREE_VIEW(self->imview)), &context.iter);
-        g_free(self->focus);
-        self->focus = NULL;
+        //g_free(self->focus);
+        //self->focus = NULL;
 
         _fcitx_im_widget_im_selection_changed(gtk_tree_view_get_selection(GTK_TREE_VIEW(self->imview)), self);
     }
